#!/usr/bin/env python3
# pip3 install bitstring

import sys
import bitstring

def madoca_rtcm(datapart):
    pos = 0
    numsat = 0
    msgnum = datapart[pos:pos+12].uint; pos += 12
    if msgnum == 1057 or msgnum == 1059 or \
       msgnum == 1061 or msgnum == 1062: # for GPS
        pos += 49
        if msgnum == 1057: pos +=  1
        numsat = datapart[pos:pos+ 6].uint; pos +=  6
        if   msgnum == 1057: # GPS orbit correction
            pos += 135 * numsat
        elif msgnum == 1059: # GPS code bias
            sumcb = 0
            for i in range(numsat):
                pos +=  6
                numcb = datapart[pos:pos+ 5].uint; pos +=  5
                sumcb += numcb
            pos += 19 * sumcb
        elif msgnum == 1061: # GPS URA
            pos += 12 * numsat
        elif msgnum == 1062: # GPS fast time correction
            pos += 28 * numsat
            pos += 1
        if pos % 8 != 0: pos += 8 - (pos % 8) # byte align
    elif msgnum == 1246 or msgnum == 1248 or \
         msgnum == 1250 or msgnum == 1251: # for QZSS
        pos += 49
        if msgnum == 1246: pos +=  1
        numsat    = datapart[pos:pos+ 4].uint; pos +=  4
        if msgnum == 1246: # QZSS orbit correction
            pos += 133 * numsat
        elif msgnum == 1248: # QZSS code bias
            sumcb = 0
            for i in range(numsat):
                pos +=  4
                numcb = datapart[pos:pos+ 5].uint; pos +=  5
                sumcb += numcb
            pos += 19 * sumcb
        elif msgnum == 1250: # QZSS URA
            pos += 10 * numsat
        elif msgnum == 1251: # QZSS fast time correction
            pos += 26 * numsat
        if pos % 8 != 0: pos += 8 - (pos % 8) # byte align
    elif msgnum == 1063 or msgnum == 1065 or \
         msgnum == 1067 or msgnum == 1068: # for GLONASS
        pos += 46
        if msgnum == 1063: pos +=  1
        numsat = datapart[pos:pos+ 6].uint; pos +=  6
        if msgnum == 1063: # GLONASS orbit correction
            pos += 134 * numsat
        elif msgnum == 1065: # GLONASS code bias
            sumcb = 0
            for i in range(numsat):
                pos +=  5
                numcb = datapart[pos:pos+ 5].uint; pos +=  5
                sumcb += numcb
            pos += 19 * sumcb
        elif msgnum == 1067: # GLONASS URA
            pos += 11 * numsat
        elif msgnum == 1068: # GLONASS fast time correction
            pos += 27 * numsat
        if pos % 8 != 0: pos += 8 - (pos % 8) # byte align
    elif msgnum == 0:
        return msgnum, numsat, b'', bitstring.BitArray('')
    else:
        print("drop {} bit: {}".format(len(datapart), datapart.bin))
        return msgnum, numsat, b'', bitstring.BitArray('')
    rtcm = datapart[0:pos]; del datapart[0:pos]
    return msgnum, numsat, rtcm.tobytes(), datapart

print ("{} {:13s}{} {:4s} {:5s} {}"
    .format("PRN", "facility:n", "A", "week", "time", "RTCM msg (satno), ..."))
sync = [b'0x00' for i in range (4)]
while True:
    b = sys.stdin.buffer.read(1)
    if not b: break
    sync = sync[1:4] + [b]
    if sync != [b'\x1a', b'\xcf', b'\xfc', b'\x1d']: continue
    prn  = int.from_bytes (sys.stdin.buffer.read(  1), 'big')
    mtid = int.from_bytes (sys.stdin.buffer.read(  1), 'big')
    data =                 sys.stdin.buffer.read(212)
    rs   =                 sys.stdin.buffer.read( 32)
    vendor   = "MADOCA" if (mtid >> 5) == 0b001 else ""
    facility = "Kobe" if (mtid >> 4) & 1 else "Hitachi-Ota"
    facility += ":" + str((mtid >> 3) &1)
    datapart = bitstring.BitArray (data)
    alert = datapart[0: 1].uint; del datapart[0: 1]
    tow   = datapart[0:20].uint; del datapart[0:20]
    wn    = datapart[0:13].uint; del datapart[0:13]
    message = ''
    while 0 < len(datapart):
        msgnum, numsat, rtcm_data, datapart = madoca_rtcm (datapart)
        if (rtcm_data == b''): break
        message += str (msgnum) + '(' + str (numsat) + ') '
    print ("{} {:13s}{} {} {} {}".format(prn, facility,
        "*" if alert else " ", wn, tow, message))
